"""
Tools for manipulation of XML properties.
"""

import random
import string

from lxml import etree

from dump2polarion import utils
from dump2polarion.exceptions import Dump2PolarionException

_NOT_EXPECTED_FORMAT_MSG = "XML file is not in expected format"


def _set_property(xml_root, name, value, properties=None):
    """Sets property to specified value."""
    if properties is None:
        properties = xml_root.find("properties")

    for prop in properties:
        if prop.get("name") == name:
            prop.set("value", utils.get_unicode_str(value))
            break
    else:
        etree.SubElement(
            properties, "property", {"name": name, "value": utils.get_unicode_str(value)}
        )


def _get_testrun_properties(xml_root):
    if xml_root.tag in ("testcases", "requirements"):
        return None
    if xml_root.tag != "testsuites":
        raise Dump2PolarionException(
            "{} {}".format(_NOT_EXPECTED_FORMAT_MSG, "- missing <testsuites>")
        )
    properties = xml_root.find("properties")
    if properties is None:
        raise Dump2PolarionException("Failed to find <properties> in the XML file")
    return properties


def xunit_fill_testrun_id(xml_root, testrun_id):
    """Adds the polarion-testrun-id property when it's missing."""
    properties = _get_testrun_properties(xml_root)
    if properties is None:
        return
    for prop in properties:
        if prop.get("name") == "polarion-testrun-id":
            break
    else:
        if not testrun_id:
            raise Dump2PolarionException(
                "Failed to submit results to Polarion - missing testrun id"
            )
        etree.SubElement(
            properties,
            "property",
            {"name": "polarion-testrun-id", "value": utils.get_unicode_str(testrun_id)},
        )


def xunit_fill_testrun_title(xml_root, testrun_title):
    """Adds the polarion-testrun-id property when it's missing."""
    properties = _get_testrun_properties(xml_root)
    if properties is None:
        return
    _set_property(xml_root, "polarion-testrun-title", testrun_title, properties)


def generate_response_property(name=None, value=None):
    """Generates response property."""
    name = name or "dump2polarion"
    value = value or "".join(random.sample(string.ascii_lowercase, 12))
    return (name, value)


def _fill_testsuites_response_property(xml_root, name, value):
    """Returns testsuites response property and fills it if missing."""
    properties = xml_root.find("properties")
    for prop in properties:
        prop_name = prop.get("name", "")
        if "polarion-response-" in prop_name:
            offset = len("polarion-response-")
            response_property = (prop_name[offset:], utils.get_unicode_str(prop.get("value")))
            break
    else:
        prop_name = "polarion-response-{}".format(name)
        etree.SubElement(
            properties, "property", {"name": prop_name, "value": utils.get_unicode_str(value)}
        )
        response_property = (name, value)

    return response_property


def _fill_non_testsuites_response_property(xml_root, name, value):
    """Returns testcases/requirements response property and fills it if missing."""
    properties = xml_root.find("response-properties")
    if properties is None:
        properties = etree.Element("response-properties")
        # response properties needs to be on top!
        xml_root.insert(0, properties)
    for prop in properties:
        if prop.tag == "response-property":
            prop_name = prop.get("name")
            prop_value = prop.get("value")
            if prop_name and prop_value:
                response_property = (prop_name, utils.get_unicode_str(prop_value))
            break
    else:
        etree.SubElement(properties, "response-property", {"name": name, "value": value})
        response_property = (name, value)

    return response_property


def fill_response_property(xml_root, name=None, value=None):
    """Returns response property and fills it if missing."""
    name, value = generate_response_property(name, value)
    response_property = None

    if xml_root.tag == "testsuites":
        response_property = _fill_testsuites_response_property(xml_root, name, value)
    elif xml_root.tag in ("testcases", "requirements"):
        response_property = _fill_non_testsuites_response_property(xml_root, name, value)
    else:
        raise Dump2PolarionException(_NOT_EXPECTED_FORMAT_MSG)

    return response_property


def remove_response_property(xml_root):
    """Removes response properties if exist."""
    if xml_root.tag == "testsuites":
        properties = xml_root.find("properties")
        resp_properties = []
        for prop in properties:
            prop_name = prop.get("name", "")
            if "polarion-response-" in prop_name:
                resp_properties.append(prop)
        for resp_property in resp_properties:
            properties.remove(resp_property)
    elif xml_root.tag in ("testcases", "requirements"):
        resp_properties = xml_root.find("response-properties")
        if resp_properties is not None:
            xml_root.remove(resp_properties)
    else:
        raise Dump2PolarionException(_NOT_EXPECTED_FORMAT_MSG)


def remove_property(xml_root, partial_name):
    """Removes properties if exist."""
    if xml_root.tag in ("testsuites", "testcases", "requirements"):
        properties = xml_root.find("properties")
        remove_properties = []
        for prop in properties:
            prop_name = prop.get("name", "")
            if partial_name in prop_name:
                remove_properties.append(prop)
        for rem_prop in remove_properties:
            properties.remove(rem_prop)
    else:
        raise Dump2PolarionException(_NOT_EXPECTED_FORMAT_MSG)


def set_lookup_method(xml_root, value):
    """Changes lookup method."""
    if xml_root.tag == "testsuites":
        _set_property(xml_root, "polarion-lookup-method", value)
    elif xml_root.tag in ("testcases", "requirements"):
        _set_property(xml_root, "lookup-method", value)
    else:
        raise Dump2PolarionException(_NOT_EXPECTED_FORMAT_MSG)


def set_dry_run(xml_root, value=True):
    """Sets dry-run so records are not updated, only log file is produced."""
    value_str = str(value).lower()
    assert value_str in ("true", "false")
    if xml_root.tag == "testsuites":
        _set_property(xml_root, "polarion-dry-run", value_str)
    elif xml_root.tag in ("testcases", "requirements"):
        _set_property(xml_root, "dry-run", value_str)
    else:
        raise Dump2PolarionException(_NOT_EXPECTED_FORMAT_MSG)



"""
Import data using correct tools.
"""

import os

from dump2polarion.results import dbtools
from dump2polarion.exceptions import Dump2PolarionException


def _get_importer(input_file):
    """Selects importer based on input file type."""
    __, ext = os.path.splitext(input_file)
    ext = ext.lower()

    if "ostriz" in input_file:
        from dump2polarion.results import ostriztools

        importer = ostriztools.import_ostriz
    elif ext == ".xml":
        # expect junit-report from pytest
        from dump2polarion.results import junittools

        importer = junittools.import_junit
    elif ext == ".csv":
        from dump2polarion.results import csvtools

        importer = csvtools.import_csv
    elif ext in dbtools.SQLITE_EXT:
        importer = dbtools.import_sqlite
    elif ext == ".json":
        from dump2polarion.results import jsontools

        importer = jsontools.import_json
    else:
        raise Dump2PolarionException("Cannot recognize type of input data, add file extension.")

    return importer


def import_results(input_file, **kwargs):
    """Imports the input file."""
    return _get_importer(input_file)(input_file, **kwargs)



"""
Creates a Testcase XML file for submitting to the Polarion Importer.

Example of input testcases_data:
testcases_data = [
    {
        "id": "ITEM01",
        "title": "test_manual",
        "description": "Manual tests with all supported fields.",
        "approver-ids": "bossman mkourim:approved",
        "assignee": "mkourim",
        "dueDate": "2018-09-30",
        "initialEstimate": "1/4h",
        "caseautomation": "manualonly",
        "caseimportance": "high",
        "caselevel": "component",
        "caseposneg": "positive",
        "testtype": "functional",
        "subtype1": "-",
        "subtype2": "-",
        "upstream": "yes",
        "tags": "tag1, tag2",
        "setup": "Do this first",
        "teardown": "Clean up",
        "automation_script": "https://gitlab.com/foo",
        "testSteps": ["step1", "step2"],
        "expectedResults": ["result1", "result2"],
        "linkedWorkItems": "ITEM01",
        "status": "proposed",
    },
    {
        "title": "test_minimal_param",
        "params": ["param1", "param2"],
        "linkedWorkItems": [{"id": "ITEM01", "role": "derived_from"}],
    },
]
"""

import datetime
import logging
import re
import urllib

from lxml import etree

from dump2polarion import utils
from dump2polarion.exceptions import Dump2PolarionException, NothingToDoException
from dump2polarion.exporters import transform_projects

# pylint: disable=invalid-name
logger = logging.getLogger(__name__)


class TestcaseTransform:
    """Transforms testcase data and fills in default keys and values."""

    TESTCASE_DATA = {
        "approver-ids": None,
        "assignee-id": None,
        "due-date": None,
        "id": None,
        "initial-estimate": None,
        "status-id": None,
    }

    FIELD_MAPPING = {
        "assignee-id": "assignee",
        "due-date": "dueDate",
        "initial-estimate": "initialEstimate",
        "status-id": "status",
        "linked-items": "linkedWorkItems",
    }

    CUSTOM_FIELDS = {
        "arch": None,
        "automation_script": None,
        "caseautomation": "automated",
        "casecomponent": None,
        "caseimportance": "high",
        "caselevel": "component",
        "caseposneg": "positive",
        "customerscenario": None,
        "endsin": None,
        "legacytest": None,
        "multiproduct": None,
        "reqverify": None,
        "setup": None,
        "startsin": None,
        "subcomponent": None,
        "subtype1": "-",
        "subtype2": "-",
        "tags": None,
        "teardown": None,
        "testtier": None,
        "testtype": "functional",
        "upstream": None,
    }

    def __init__(self, config, transform_func=None):
        self.config = config
        self._transform_func = transform_func or transform_projects.get_testcases_transform(config)

        default_fields = self.config.get("default_fields") or {}
        default_fields = {k: utils.get_unicode_str(v) for k, v in default_fields.items() if v}
        self.default_fields = utils.sorted_dict(default_fields)
        self.repo_address = self._get_full_repo_address(self.config.get("repo_address"))

    @staticmethod
    def _get_full_repo_address(repo_address):
        """Makes sure the repo address is complete path in repository.

        >>> TestcaseTransform._get_full_repo_address("https://gitlab.com/somerepo")
        'https://gitlab.com/somerepo/blob/master/'
        >>> TestcaseTransform._get_full_repo_address("https://github.com/otherrepo/blob/branch/")
        'https://github.com/otherrepo/blob/branch/'
        >>> TestcaseTransform._get_full_repo_address(None)
        """
        if not repo_address:
            return None

        if "/blob/" not in repo_address:
            # the master here should probably link the latest "commit" eventually
            repo_address = "{}/blob/master".format(repo_address)

        # make sure the / is present at the end of address
        repo_address = "{}/".format(repo_address.rstrip("/ "))

        return repo_address

    def _fill_project_defaults(self, testcase_data):
        filled = self.default_fields.copy()
        filled.update(testcase_data)
        return filled

    def _fill_automation_repo(self, testcase_data):
        automation_script = testcase_data.get("automation_script")
        if not (self.repo_address and automation_script) or automation_script.startswith("http"):
            return testcase_data

        testcase_data["automation_script"] = urllib.parse.urljoin(
            self.repo_address, automation_script
        )
        return testcase_data

    def _run_transform_func(self, testcase_data):
        """Calls transform function on testcase data."""
        if self._transform_func:
            testcase_data = self._transform_func(testcase_data)
        return testcase_data or None

    def _fill_polarion_fields(self, testcase_data):
        """Sets importer field value from polarion field if available."""
        for importer_field, polarion_field in self.FIELD_MAPPING.items():
            polarion_value = testcase_data.get(polarion_field)
            xml_value = testcase_data.get(importer_field)
            if polarion_value and not xml_value:
                testcase_data[importer_field] = polarion_value
        return testcase_data

    def _fill_defaults(self, testcase_data):
        for defaults in self.TESTCASE_DATA, self.CUSTOM_FIELDS:
            for key, value in defaults.items():
                if value and not testcase_data.get(key):
                    testcase_data[key] = value
        return testcase_data

    def transform(self, testcase_data):
        """Transforms testcase data."""
        testcase_data = self._fill_project_defaults(testcase_data)
        testcase_data = self._fill_automation_repo(testcase_data)
        testcase_data = self._fill_polarion_fields(testcase_data)
        testcase_data = self._run_transform_func(testcase_data)
        if not testcase_data:
            return None

        testcase_data = self._fill_defaults(testcase_data)
        return testcase_data


class TestcaseExport:
    """Exports testcases data into XML representation."""

    def __init__(self, testcases_data, config, transform_func=None):
        self.testcases_data = testcases_data
        self.config = config
        self._lookup_prop = ""
        self.testcases_transform = TestcaseTransform(config, transform_func)

        self.known_custom_fields = set(self.testcases_transform.CUSTOM_FIELDS)
        self.known_custom_fields.update(self.config.get("custom_fields") or ())

        self._compiled_whitelist = None
        self._compiled_blacklist = None
        if self.config.get("whitelisted_tests"):
            self._compiled_whitelist = re.compile(
                "(" + ")|(".join(self.config.get("whitelisted_tests")) + ")"
            )
        if self.config.get("blacklisted_tests"):
            self._compiled_blacklist = re.compile(
                "(" + ")|(".join(self.config.get("blacklisted_tests")) + ")"
            )

    def _top_element(self):
        """Returns top XML element."""
        attrs = {"project-id": self.config["polarion-project-id"]}
        top = etree.Element("testcases", attrs)
        return top

    def _properties_element(self, parent_element):
        """Returns properties XML element."""
        testcases_properties = etree.SubElement(parent_element, "properties")

        testcases_properties_conf = self.config.get("testcase_import_properties") or {}
        for name, value in sorted(testcases_properties_conf.items()):
            if name == "lookup-method":
                lookup_prop = str(value).lower()
                if lookup_prop not in ("id", "name", "custom"):
                    raise Dump2PolarionException(
                        "Invalid value '{}' for the 'lookup-method' property".format(str(value))
                    )
                self._lookup_prop = lookup_prop
            else:
                etree.SubElement(
                    testcases_properties, "property", {"name": name, "value": str(value)}
                )

        return testcases_properties

    def _fill_lookup_prop(self, testcases_properties):
        """Fills the polarion-lookup-method property."""
        if not self._lookup_prop:
            raise Dump2PolarionException("Failed to set the 'polarion-lookup-method' property")

        etree.SubElement(
            testcases_properties, "property", {"name": "lookup-method", "value": self._lookup_prop}
        )

    def _set_lookup_prop(self, testcase_data):
        """Set lookup property based on processed testcases if not configured."""
        if self._lookup_prop:
            return

        if testcase_data.get("id"):
            self._lookup_prop = "id"
        elif testcase_data.get("title"):
            self._lookup_prop = "name"
        else:
            return

        logger.debug("Setting lookup method for testcases to `%s`", self._lookup_prop)

    def _check_lookup_prop(self, testcase_data):
        """Checks that selected lookup property can be used for this testcase."""
        if not self._lookup_prop:
            return False

        if not testcase_data.get("id") and self._lookup_prop != "name":
            return False
        if not testcase_data.get("title") and self._lookup_prop == "name":
            return False
        return True

    def _get_testcase_id(self, testcase_data):
        """Returns testcase id when possible."""
        testcase_id = testcase_data.get("id")
        if testcase_id:
            return testcase_id
        if self._lookup_prop != "name":
            return None
        return testcase_data.get("title")

    def _classify_data(self, testcase_data):
        attrs, custom_fields = {}, {}

        for key, value in testcase_data.items():
            if not value:
                continue
            if key in self.testcases_transform.TESTCASE_DATA:
                attrs[key] = value
            elif key in self.known_custom_fields:
                custom_fields[key] = value

        return attrs, custom_fields

    @staticmethod
    def _add_test_steps(parent, testcase_data):
        steps = testcase_data.get("testSteps")
        results = testcase_data.get("expectedResults")
        params = testcase_data.get("params") or ()
        test_steps = etree.SubElement(parent, "test-steps")

        if steps and testcase_data["caseautomation"] != "automated":
            for index, step in enumerate(steps):
                test_step = etree.SubElement(test_steps, "test-step")
                test_step_col = etree.SubElement(test_step, "test-step-column", id="step")
                test_step_col.text = utils.get_unicode_str(step)

                test_res_col = etree.SubElement(test_step, "test-step-column", id="expectedResult")
                try:
                    test_res_col.text = utils.get_unicode_str(results[index])
                except IndexError:
                    test_res_col.text = ""
        else:
            test_step = etree.SubElement(test_steps, "test-step")
            test_step_col = etree.SubElement(test_step, "test-step-column", id="step")
            for param in params:
                param_el = etree.Element("parameter", name=param, scope="local")
                test_step_col.append(param_el)

    @staticmethod
    def _add_linked_items(parent, testcase_data):
        linked_items = testcase_data.get("linked-items") or testcase_data.get("linked-work-items")
        if not linked_items:
            return
        if isinstance(linked_items, str) and "," in linked_items:
            # multiple unprocessed linked items (should be list already), skip them
            return
        if isinstance(linked_items, (dict, str)):
            linked_items = [linked_items]

        linked_work_items = etree.SubElement(parent, "linked-work-items")
        for work_item in linked_items:
            if isinstance(work_item, str):
                work_item_id = work_item
                work_item_role = "verifies"
            else:
                work_item_id = work_item.get("id")
                work_item_role = work_item.get("role") or "verifies"

            if not work_item_id:
                continue

            work_item_el = etree.SubElement(linked_work_items, "linked-work-item")
            work_item_el.attrib["workitem-id"] = work_item_id
            work_item_el.attrib["role-id"] = work_item_role

            lookup_method = testcase_data.get("linked-items-lookup-method")
            if lookup_method in ("name", "id"):
                work_item_el.attrib["lookup-method"] = lookup_method

    @staticmethod
    def _fill_custom_fields(parent, custom_fields):
        if not custom_fields:
            return

        custom_fields_el = etree.SubElement(parent, "custom-fields")
        for field, content in custom_fields.items():
            etree.SubElement(
                custom_fields_el,
                "custom-field",
                utils.sorted_dict({"id": field, "content": utils.get_unicode_str(content)}),
            )

    def _is_whitelisted(self, nodeid):
        """Checks if the nodeid is whitelisted."""
        if not nodeid:
            return True
        if self._compiled_whitelist and self._compiled_whitelist.search(nodeid):
            return True
        if self._compiled_blacklist and self._compiled_blacklist.search(nodeid):
            return False
        return True

    def _testcase_element(self, parent_element, testcase_data):
        """Adds testcase XML element."""
        nodeid = testcase_data.get("nodeid")
        if not self._is_whitelisted(nodeid):
            logger.debug("Skipping blacklisted node: %s", nodeid)
            return

        testcase_data = self.testcases_transform.transform(testcase_data)
        if not testcase_data:
            return

        testcase_title = testcase_data.get("title")
        self._set_lookup_prop(testcase_data)
        if not self._check_lookup_prop(testcase_data):
            logger.warning(
                "Skipping testcase `%s`, data missing for selected lookup method",
                testcase_data.get("id") or testcase_title,
            )
            return

        # make sure that ID is set even for "name" lookup method
        testcase_data["id"] = self._get_testcase_id(testcase_data)

        attrs, custom_fields = self._classify_data(testcase_data)

        # For testing purposes, the order of fields in resulting XML
        # needs to be always the same.
        attrs = utils.sorted_dict(attrs)
        custom_fields = utils.sorted_dict(custom_fields)

        testcase = etree.SubElement(parent_element, "testcase", attrs)

        title_el = etree.SubElement(testcase, "title")
        title_el.text = utils.get_unicode_str(testcase_title)

        description = testcase_data.get("description")
        if description:
            description_el = etree.SubElement(testcase, "description")
            description_el.text = utils.get_unicode_str(description)

        self._add_test_steps(testcase, testcase_data)
        self._fill_custom_fields(testcase, custom_fields)
        self._add_linked_items(testcase, testcase_data)

    def _fill_testcases(self, parent_element):
        if not self.testcases_data:
            raise NothingToDoException("Nothing to export")
        for testcase_data in self.testcases_data:
            self._testcase_element(parent_element, testcase_data)

    def export(self):
        """Returns testcases XML."""
        top = self._top_element()
        properties = self._properties_element(top)
        self._fill_testcases(top)
        self._fill_lookup_prop(properties)
        return utils.prettify_xml(top)

    @staticmethod
    def write_xml(xml, output_file=None):
        """Outputs the XML content into a file."""
        gen_filename = "testcases-{:%Y%m%d%H%M%S}.xml".format(datetime.datetime.now())
        utils.write_xml(xml, output_loc=output_file, filename=gen_filename)



"""
Helper functions for handling JSON data.
"""

import json

from dump2polarion.exceptions import Dump2PolarionException
from dump2polarion.exporters import xunit_exporter


def _load_json(json_filename):
    with open(json_filename, encoding="utf-8") as input_json:
        return json.load(input_json)


def import_pytest_collect(json_filename):
    """Reads the content of the JSON file produced by pytest-polarion-collect file."""
    try:
        results = _load_json(json_filename)["results"]
    except Exception as err:
        raise Dump2PolarionException("Cannot load results from {}: {}".format(json_filename, err))

    return xunit_exporter.ImportedData(results=results, testrun=None)


# pylint: disable=unused-argument
def import_json(json_filename, **kwargs):
    """Reads the content of the JSON file."""
    # results from pytest-polarion-collect are the only ones supported so far
    return import_pytest_collect(json_filename)



# pylint: disable=missing-docstring,redefined-outer-name,no-self-use

import os

from mock import patch

from dump2polarion import csv2sqlite_cli
from dump2polarion.exporters.xunit_exporter import XunitExport
from dump2polarion.results import dbtools
from tests import conf


class TestCSV2sqliteCLI:
    def test_get_args(self):
        args = csv2sqlite_cli.get_args(["-i", "foo", "-o", "bar"])
        assert args.input_file == "foo"
        assert args.output_file == "bar"
        assert args.log_level is None

    def test_e2e_ok(self, config_prop, tmpdir):
        input_file = os.path.join(conf.DATA_PATH, "workitems_ids.csv")
        db_file = os.path.join(str(tmpdir), "workitems_copy.sqlite3")
        args = ["-i", input_file, "-o", db_file]
        with patch("dump2polarion.csv2sqlite_cli.utils.init_log"):
            retval = csv2sqlite_cli.main(args)
        assert retval == 0

        records = dbtools.import_sqlite(db_file)
        exporter = XunitExport("5_8_0_17", records, config_prop)
        complete = exporter.export()
        fname = "complete_transform.xml"
        with open(os.path.join(conf.DATA_PATH, fname), encoding="utf-8") as input_xml:
            parsed = input_xml.read()
        assert complete == parsed

    def test_e2e_invalid_csv(self, captured_log):
        non_file = "nonexistent.csv"
        args = ["-i", non_file, "-o", non_file]
        retval = csv2sqlite_cli.main(args)
        assert retval == 1
        assert "No such file or directory" in captured_log.getvalue()

    def test_e2e_not_csv(self, captured_log):
        non_file = "nonexistent.txt"
        args = ["-i", non_file, "-o", non_file]
        retval = csv2sqlite_cli.main(args)
        assert retval == 1
        assert "is in CSV format" in captured_log.getvalue()



"""
Find out info about work items in Polarion by parsing logs produced by Importers.

Find out what items:
  * exists (and can be updated)
  * are new (or need to be created)
  * have duplicates
  * Polarion IDs of existing items

Logs can be produced by submitting XMLs with dry-only.
"""

import os
import re

from dump2polarion.exceptions import Dump2PolarionException


class LogItem:
    """Represents one work item record in a log file."""

    # pylint: disable=redefined-builtin
    def __init__(self, name, id, custom_id):
        self.name = name
        self.id = id
        self.custom_id = custom_id

    def __repr__(self):
        return "<LogItem {}>".format(self.name)


class ParsedLog:
    """Outcome of log parsing."""

    def __init__(self, log_type, new_items, existing_items, duplicate_items):
        self.log_type = log_type
        self.new_items = new_items
        self.existing_items = existing_items
        self.duplicate_items = duplicate_items

    def __len__(self):
        return 1 if (self.new_items or self.existing_items or self.duplicate_items) else 0

    def __repr__(self):
        return "<ParsedLog {}>".format(self.log_type)


class XUnitParser:
    """Parser for XUnit logs."""

    RESULT_SEARCH = re.compile(r"Work item: '(test_[^']+|[A-Z][^']+)' \(([^)]+)\)$")
    RESULT_WARN_SEARCH = re.compile(r" '(test_[^']+|[A-Z][^']+)'\.$")
    RESULT_WARN_SEARCH_CUSTOM = re.compile(r" '(test_[^']+|[A-Z][^']+)' \(([^)]+)\)\.$")

    def __init__(self, fp, log_file):
        self.fp = fp
        self.log_file = log_file

    def get_result(self, line):
        """Gets work item name and id."""
        res = self.RESULT_SEARCH.search(line)
        try:
            name, ids = res.group(1), res.group(2)
        except (AttributeError, IndexError):
            return None

        ids = ids.split("/")
        tc_id = ids[0]
        try:
            custom_id = ids[1]
        except IndexError:
            custom_id = None
        return LogItem(name, tc_id, custom_id)

    def get_result_warn(self, line):
        """Gets work item name of item that was not successfully imported."""
        res = self.RESULT_WARN_SEARCH.search(line)
        try:
            return LogItem(res.group(1), None, None)
        except (AttributeError, IndexError):
            pass

        # try again with custom ID
        res = self.RESULT_WARN_SEARCH_CUSTOM.search(line)
        try:
            return LogItem(res.group(1), None, res.group(2))
        except (AttributeError, IndexError):
            return None

    def parse(self):
        """Parse log file produced by the XUnit Iporter."""
        existing_items = []
        duplicate_items = []
        new_items = []
        for line in self.fp:
            line = line.strip()
            if "Work item: " in line:
                existing_it = self.get_result(line)
                if existing_it:
                    existing_items.append(existing_it)
            elif "Unable to find *unique* work item" in line:
                duplicate_it = self.get_result_warn(line)
                if duplicate_it:
                    duplicate_items.append(duplicate_it)
            elif "Unable to find work item for" in line:
                new_it = self.get_result_warn(line)
                if new_it:
                    new_items.append(new_it)

        outcome = ParsedLog("xunit", new_items, existing_items, duplicate_items)

        if not outcome:
            raise Dump2PolarionException(
                "No valid data found in the log file '{}'".format(self.log_file)
            )

        return outcome


class TestcasesParser:
    """Parser for Testcase logs."""

    TESTCASE_SEARCH = re.compile(r" test case '(test_[^']+|[A-Z][^']+)' \(([^)]+)\)")
    TESTCASE_WARN_SEARCH = re.compile(r" '(test_[^']+|[A-Z][^']+)'\.$")

    def __init__(self, fp, log_file):
        self.fp = fp
        self.log_file = log_file

    def get_testcase(self, line):
        """Gets test case name and id."""
        res = self.TESTCASE_SEARCH.search(line)
        try:
            name, ids = res.group(1), res.group(2)
        except (AttributeError, IndexError):
            return None

        ids = ids.split("/")
        tc_id = ids[0]
        try:
            custom_id = ids[1]
        except IndexError:
            custom_id = None
        return LogItem(name, tc_id, custom_id)

    def get_testcase_warn(self, line):
        """Gets name of test case that was not successfully imported."""
        res = self.TESTCASE_WARN_SEARCH.search(line)
        try:
            return LogItem(res.group(1), None, None)
        except (AttributeError, IndexError):
            return None

    def parse(self):
        """Parse log file produced by the Test Case Importer."""
        existing_items = []
        duplicate_items = []
        new_items = []
        for line in self.fp:
            line = line.strip()
            if "Updated test case" in line:
                existing_it = self.get_testcase(line)
                if existing_it:
                    existing_items.append(existing_it)
            elif "Found multiple work items with the title" in line:
                duplicate_it = self.get_testcase_warn(line)
                if duplicate_it:
                    duplicate_items.append(duplicate_it)
            elif "Created test case" in line:
                new_it = self.get_testcase(line)
                if new_it:
                    new_items.append(new_it)

        outcome = ParsedLog("testcase", new_items, existing_items, duplicate_items)

        if not outcome:
            raise Dump2PolarionException(
                "No valid data found in the log file '{}'".format(self.log_file)
            )

        return outcome


class RequirementsParser:
    """Parser for Requirement logs."""

    REQ_SEARCH = re.compile(r" requirement '([a-zA-Z][^']+)' \(([^)/]+)")
    REQ_WARN_SEARCH = re.compile(r" '([^']+)'\.$")

    def __init__(self, fp, log_file):
        self.fp = fp
        self.log_file = log_file

    def get_requirement(self, line):
        """Gets requirement name and id."""
        res = self.REQ_SEARCH.search(line)
        try:
            name, tc_id = res.group(1), res.group(2)
        except (AttributeError, IndexError):
            return None

        return LogItem(name, tc_id, None)

    def get_requirement_warn(self, line):
        """Gets name of test case that was not successfully imported."""
        res = self.REQ_WARN_SEARCH.search(line)
        try:
            return LogItem(res.group(1), None, None)
        except (AttributeError, IndexError):
            return None

    def parse(self):
        """Parse log file produced by the Requirements Importer."""
        existing_items = []
        duplicate_items = []
        new_items = []
        for line in self.fp:
            line = line.strip()
            if "Updated requirement" in line:
                existing_it = self.get_requirement(line)
                if existing_it:
                    existing_items.append(existing_it)
            elif "Found multiple work items with the title" in line:
                duplicate_it = self.get_requirement_warn(line)
                if duplicate_it:
                    duplicate_items.append(duplicate_it)
            elif "Created requirement" in line:
                new_it = self.get_requirement(line)
                if new_it:
                    new_items.append(new_it)

        outcome = ParsedLog("requirement", new_items, existing_items, duplicate_items)

        if not outcome:
            raise Dump2PolarionException(
                "No valid data found in the log file '{}'".format(self.log_file)
            )

        return outcome


def parse(log_file):
    """Parse log file."""
    with open(os.path.expanduser(log_file), encoding="utf-8") as input_file:
        for line in input_file:
            if "Starting import of XUnit results" in line:
                obj = XUnitParser
                break
            if "Starting import of test cases" in line:
                obj = TestcasesParser
                break
            if "Starting import of requirements" in line:
                obj = RequirementsParser
                break
        else:
            raise Dump2PolarionException(
                "No valid data found in the log file '{}'".format(log_file)
            )

        return obj(input_file, log_file).parse()



"""
Creates a Requirement XML file for submitting to the Polarion Importer.

Example of input requirements_data:
requirements_data = [
    {
        "title": "requirement_complete",
        "description": "Complete Requirement",
        "approver-ids": "mkourim:approved",
        "assignee": "mkourim",
        "category-ids": "category_id1, category_id2",
        "dueDate": "2018-09-30",
        "plannedIn": "planned_id1, planned_id2",
        "initialEstimate": "1/4h",
        "priority": "high",
        "severity": "should_have",
        "status": "status_id",
        "reqtype": "functional",
    },
    {
        "title": "requirement_minimal",
    },
]
"""

import datetime
import logging

from lxml import etree

from dump2polarion import utils
from dump2polarion.exceptions import Dump2PolarionException, NothingToDoException
from dump2polarion.exporters import transform_projects

# pylint: disable=invalid-name
logger = logging.getLogger(__name__)


class RequirementTransform:
    """Transforms requirement data and fills in default keys and values."""

    REQ_DATA = {
        "approver-ids": None,
        "assignee-id": None,
        "category-ids": None,
        "due-date": None,
        "initial-estimate": None,
        "planned-in-ids": None,
        "priority-id": "high",
        "severity-id": "should_have",
        "status-id": None,
    }

    FIELD_MAPPING = {
        "assignee-id": "assignee",
        "due-date": "dueDate",
        "initial-estimate": "initialEstimate",
        "planned-in-ids": "plannedIn",
        "priority-id": "priority",
        "severity-id": "severity",
        "status-id": "status",
    }

    CUSTOM_FIELDS = {"reqtype": "functional"}

    def __init__(self, config, transform_func=None):
        self.config = config
        self._transform_func = transform_func or transform_projects.get_requirements_transform(
            config
        )

        default_fields = self.config.get("requirements_default_fields") or {}
        default_fields = {k: utils.get_unicode_str(v) for k, v in default_fields.items() if v}
        self.default_fields = utils.sorted_dict(default_fields)

    def _fill_project_defaults(self, testcase_data):
        filled = self.default_fields.copy()
        filled.update(testcase_data)
        return filled

    def _run_transform_func(self, result):
        """Calls transform function on result."""
        if self._transform_func:
            result = self._transform_func(result)
        return result or None

    def _fill_polarion_fields(self, req_data):
        """Sets importer field value from polarion field if available."""
        for importer_field, polarion_field in self.FIELD_MAPPING.items():
            polarion_value = req_data.get(polarion_field)
            xml_value = req_data.get(importer_field)
            if polarion_value and not xml_value:
                req_data[importer_field] = polarion_value
        return req_data

    def _fill_defaults(self, req_data):
        for defaults in self.REQ_DATA, self.CUSTOM_FIELDS:
            for key, value in defaults.items():
                if value and not req_data.get(key):
                    req_data[key] = value
        return req_data

    def transform(self, req_data):
        """Transforms requirement data."""
        req_data = self._fill_project_defaults(req_data)
        req_data = self._fill_polarion_fields(req_data)
        req_data = self._run_transform_func(req_data)
        if not req_data:
            return None

        title = req_data.get("title")
        if not title:
            logger.warning("Skipping requirement, title is missing")
            return None

        req_data = self._fill_defaults(req_data)
        return req_data


class RequirementExport:
    """Exports requirements data into XML representation."""

    def __init__(self, requirements_data, config, transform_func=None):
        self.requirements_data = requirements_data
        self.config = config
        self._lookup_prop = ""
        self.requirement_transform = RequirementTransform(config, transform_func)

        self.known_custom_fields = set(self.requirement_transform.CUSTOM_FIELDS)
        self.known_custom_fields.update(self.config.get("requirements_custom_fields") or ())

    def _top_element(self):
        """Returns top XML element."""
        attrs = {"project-id": self.config["polarion-project-id"]}
        document_relative_path = self.config.get("requirements-document-relative-path")
        if document_relative_path:
            attrs["document-relative-path"] = document_relative_path
        top = etree.Element("requirements", utils.sorted_dict(attrs))
        return top

    def _properties_element(self, parent_element):
        """Returns properties XML element."""
        requirements_properties = etree.SubElement(parent_element, "properties")

        req_properties_conf = self.config.get("requirements_import_properties") or {}
        for name, value in sorted(req_properties_conf.items()):
            if name == "lookup-method":
                lookup_prop = str(value).lower()
                if lookup_prop not in ("id", "name"):
                    raise Dump2PolarionException(
                        "Invalid value '{}' for the 'lookup-method' property".format(str(value))
                    )
                self._lookup_prop = lookup_prop
            else:
                etree.SubElement(
                    requirements_properties, "property", {"name": name, "value": str(value)}
                )

        return requirements_properties

    def _fill_lookup_prop(self, requirements_properties):
        """Fills the polarion-lookup-method property."""
        if not self._lookup_prop:
            raise Dump2PolarionException("Failed to set the 'polarion-lookup-method' property")

        etree.SubElement(
            requirements_properties,
            "property",
            {"name": "lookup-method", "value": self._lookup_prop},
        )

    def _check_lookup_prop(self, req_id):
        """Checks that selected lookup property can be used for this testcase."""
        if self._lookup_prop:
            if not req_id and self._lookup_prop == "id":
                return False
        else:
            if req_id:
                self._lookup_prop = "id"
            else:
                self._lookup_prop = "name"
        return True

    def _classify_data(self, req_data):
        attrs, custom_fields = {}, {}

        for key, value in req_data.items():
            if not value:
                continue
            conv_key = key.replace("_", "-")  # convert pythonic key_param to polarion 'key-param'
            for key_variant in (conv_key, key):
                if key_variant in self.requirement_transform.REQ_DATA:
                    attrs[key_variant] = value
                elif key_variant in self.known_custom_fields:
                    custom_fields[key_variant] = value
                if conv_key == key:
                    break

        return attrs, custom_fields

    @staticmethod
    def _fill_custom_fields(parent, custom_fields):
        if not custom_fields:
            return

        custom_fields_el = etree.SubElement(parent, "custom-fields")
        for field, content in custom_fields.items():
            etree.SubElement(
                custom_fields_el,
                "custom-field",
                utils.sorted_dict({"id": field, "content": content}),
            )

    def _requirement_element(self, parent_element, req_data):
        """Adds requirement XML element."""
        req_data = self.requirement_transform.transform(req_data)
        if not req_data:
            return

        title = req_data.get("title")

        req_id = req_data.get("id")
        if not self._check_lookup_prop(req_id):
            logger.warning(
                "Skipping requirement `%s`, data missing for selected lookup method", title
            )
            return

        attrs, custom_fields = self._classify_data(req_data)

        # For testing purposes, the order of fields in resulting XML
        # needs to be always the same.
        attrs = utils.sorted_dict(attrs)
        custom_fields = utils.sorted_dict(custom_fields)

        requirement = etree.SubElement(parent_element, "requirement", attrs)

        title_el = etree.SubElement(requirement, "title")
        title_el.text = utils.get_unicode_str(title)

        description = req_data.get("description")
        if description:
            description_el = etree.SubElement(requirement, "description")
            description_el.text = utils.get_unicode_str(description)

        self._fill_custom_fields(requirement, custom_fields)

    def _fill_requirements(self, parent_element):
        if not self.requirements_data:
            raise NothingToDoException("Nothing to export")
        for req_data in self.requirements_data:
            self._requirement_element(parent_element, req_data)

    def export(self):
        """Returns requirements XML."""
        top = self._top_element()
        properties = self._properties_element(top)
        self._fill_requirements(top)
        self._fill_lookup_prop(properties)
        return utils.prettify_xml(top)

    @staticmethod
    def write_xml(xml, output_file=None):
        """Outputs the XML content into a file."""
        gen_filename = "requirements-{:%Y%m%d%H%M%S}.xml".format(datetime.datetime.now())
        utils.write_xml(xml, output_loc=output_file, filename=gen_filename)



# pylint: disable=missing-docstring,redefined-outer-name,no-self-use

import os

import pytest
from mock import patch

from dump2polarion import utils
from dump2polarion.exceptions import Dump2PolarionException
from tests import conf


# pylint: disable=too-many-public-methods
class TestUtils:
    def test_get_unicode_char(self):
        unicode_str = utils.get_unicode_str("®")
        assert unicode_str == "®"

    def test_get_unicode_num(self):
        unicode_str = utils.get_unicode_str(42)
        assert unicode_str == "42"

    def test_get_unicode_basestring(self):
        unicode_str = utils.get_unicode_str(b"@")
        assert unicode_str == "@"

    def test_write_xml_gen(self, tmpdir):
        dirname = str(tmpdir)
        utils.write_xml("<xml />", output_loc=dirname)
        assert "output-" in os.listdir(dirname)[0]

    def test_write_xml_loc(self, tmpdir):
        dirname = str(tmpdir)
        utils.write_xml("<xml />", output_loc=os.path.join(dirname, "output123.xml"))
        assert "output123.xml" in os.listdir(dirname)[0]

    def test_write_xml_file(self, tmpdir):
        dirname = str(tmpdir)
        utils.write_xml("<xml />", filename=os.path.join(dirname, "output123.xml"))
        assert "output123.xml" in os.listdir(dirname)[0]

    def test_write_xml_root_none(self, tmpdir):
        dirname = str(tmpdir)
        with pytest.raises(Dump2PolarionException) as excinfo:
            utils.write_xml_root(None, filename=os.path.join(dirname, "output123.xml"))
        assert "No data to write" in str(excinfo.value)

    def test_write_xml_root_file(self, tmpdir):
        fname = "complete_transform_noresponse.xml"
        xml_root = utils.get_xml_root(os.path.join(conf.DATA_PATH, fname))
        dirname = str(tmpdir)
        utils.write_xml_root(xml_root, filename=os.path.join(dirname, "output123.xml"))
        assert "output123.xml" in os.listdir(dirname)[0]

    def test_write_xml_no_data(self, tmpdir):
        dirname = str(tmpdir)
        with pytest.raises(Dump2PolarionException) as excinfo:
            utils.write_xml("", filename=os.path.join(dirname, "output123.xml"))
        assert "No data to write" in str(excinfo.value)

    def test_invalid_xml_root(self):
        with pytest.raises(Dump2PolarionException) as excinfo:
            utils.get_xml_root("NONEXISTENT.xml")
        assert "Failed to parse XML file" in str(excinfo.value)

    def test_invalid_xml_str(self):
        with pytest.raises(Dump2PolarionException) as excinfo:
            utils.get_xml_root_from_str(None)
        assert "Failed to parse XML string" in str(excinfo.value)

    def test_get_session_oldauth(self, config_prop):
        if "auth_url" in config_prop:
            del config_prop["auth_url"]
        with patch("requests.Session"):
            session = utils.get_session("foo", config_prop)
        assert session.auth == "foo"

    def test_get_session_cookie_failed(self, config_prop):
        config_prop["auth_url"] = "http://example.com"
        with patch("requests.Session") as mock:
            instance = mock.return_value
            instance.post.return_value = None
            with pytest.raises(Dump2PolarionException) as excinfo:
                utils.get_session("foo", config_prop)
        assert "Cookie was not retrieved" in str(excinfo.value)

    def test_get_session_cookie_success(self, config_prop):
        config_prop["auth_url"] = "http://example.com"
        with patch("requests.Session") as mock:
            instance = mock.return_value
            instance.post.return_value = True
            session = utils.get_session("foo", config_prop)
        assert session.auth != "foo"



"""
Configuration loading.
"""

import logging
import os

import yaml
from polarion_tools_common import configuration, utils

from dump2polarion.exceptions import Dump2PolarionException

DEFAULT_CONF = os.path.join(os.path.dirname(os.path.abspath(__file__)), "polarion_tools.yaml")
PROJECT_CONF_DIRS = ("conf", ".")
PROJECT_CONF = "polarion_tools*.yaml"

URLS = {
    "testcase_taget": "import/testcase",
    "xunit_target": "import/xunit",
    "requirement_target": "import/requirement",
    "testcase_queue": "import/testcase-queue",
    "xunit_queue": "import/xunit-queue",
    "requirement_queue": "import/requirement-queue",
    "testcase_log": "import/testcase-log",
    "xunit_log": "import/xunit-log",
    "requirement_log": "import/requirement-log",
    "auth_url": "j_security_check",
}

# pylint: disable=invalid-name
logger = logging.getLogger(__name__)


def _check_config(config):
    missing = []
    for key in URLS:
        if not config.get(key):
            missing.append(key)

    # the 'auth_url' is allowed to be empty for now
    # TODO: can be removed once basic auth is discontinued on prod
    if not config.get("auth_url") and "auth_url" in config:
        missing.remove("auth_url")

    if missing:
        raise Dump2PolarionException(
            "Failed to find following keys in config file: {}\n"
            "Please see https://mojo.redhat.com/docs/DOC-1098563#config".format(", ".join(missing))
        )


def _guess_base_url(config):
    for key, value in URLS.items():
        if config.get(key):
            return config[key][: -len(value)]
    return None


def _populate_urls(config):
    base_url = config.get("polarion_url")
    if not base_url:
        base_url = _guess_base_url(config)
    if not base_url:
        return

    base_url = base_url.rstrip("/")
    for key, url in URLS.items():
        if key not in config:
            config[key] = "{}/{}".format(base_url, url)


def _set_legacy_project_id(config):
    if config.get("polarion-project-id"):
        return

    # use legacy configuration if available
    xunit_project = config.get("xunit_import_properties") or {}
    xunit_project = xunit_project.get("polarion-project-id")
    if not xunit_project:
        raise Dump2PolarionException('The "polarion-project-id" key is missing in the config file')

    config["polarion-project-id"] = xunit_project
    logger.warning(
        'Loading the "polarion-project-id" from legacy configuration under'
        " xunit_import_properties instead of from top level"
    )


def _set_legacy_custom_fields(config):
    if config.get("custom_fields"):
        return

    # use legacy configuration if available
    custom_fields = config.get("docstrings") or {}
    custom_fields = custom_fields.get("custom_fields")
    if custom_fields:
        config["custom_fields"] = custom_fields
        logger.warning(
            'Loading the "custom_fields" from legacy configuration under "docstrings"'
            " instead of from top level"
        )


def _get_default_conf():
    with open(DEFAULT_CONF, encoding="utf-8") as input_file:
        config_settings = yaml.safe_load(input_file)

    logger.debug("Default config loaded from %s", DEFAULT_CONF)

    return config_settings


def _get_user_conf(config_file):
    try:
        with open(os.path.expanduser(config_file), encoding="utf-8") as input_file:
            config_settings = yaml.safe_load(input_file)
    except OSError:
        raise Dump2PolarionException("Cannot open config file {}".format(config_file))

    logger.info("Config loaded from %s", config_file)

    return config_settings


def get_config(config_file=None, config_values=None, load_project_conf=True):
    """Loads config file and returns its content."""
    config_values = config_values or {}

    default_conf = _get_default_conf()
    user_conf = _get_user_conf(config_file) if config_file else {}
    # load project configuration only when user configuration was not specified
    project_conf = {} if user_conf or not load_project_conf else configuration.get_config()

    if not (user_conf or project_conf or config_values):
        if load_project_conf:
            raise Dump2PolarionException(
                "Failed to find configuration file for the project "
                "and no configuration file or values passed."
            )
        raise Dump2PolarionException("No configuration file or values passed.")

    # merge configuration
    config_settings = {}
    config_settings.update(default_conf)
    utils.merge_dicts(config_settings, user_conf)
    utils.merge_dicts(config_settings, project_conf)
    utils.merge_dicts(config_settings, config_values)

    _populate_urls(config_settings)
    _set_legacy_project_id(config_settings)
    _set_legacy_custom_fields(config_settings)
    _check_config(config_settings)

    return config_settings



"""
Utils for dump2polarion.
"""

import datetime
import logging
import os
import random
import re
import string
from collections import OrderedDict

import requests
import urllib3
from lxml import etree
from polarion_tools_common import utils

from dump2polarion.exceptions import Dump2PolarionException

urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)

# pylint: disable=invalid-name
logger = logging.getLogger(__name__)

NO_BLANKS_PARSER = etree.XMLParser(remove_blank_text=True)
# from https://stackoverflow.com/a/25920392
VALID_XML_RE = re.compile("[^\u0020-\uD7FF\u0009\u000A\u000D\uE000-\uFFFD\U00010000-\U0010FFFF]+")


def get_unicode_str(obj):
    """Makes sure obj is a valid XML unicode string."""
    return VALID_XML_RE.sub("", utils.get_unicode_str(obj))


def init_log(log_level):
    """Initializes logging."""
    log_level = log_level or "INFO"
    logging.basicConfig(
        format="%(name)s:%(levelname)s:%(message)s",
        level=getattr(logging, log_level.upper(), logging.INFO),
    )


def _get_filename(output_loc=None, filename=None):
    filename = filename or "output-{:%Y%m%d%H%M%S}-{}.xml".format(
        datetime.datetime.now(), "".join(random.sample(string.ascii_lowercase, 5))
    )
    if output_loc:
        filename_fin = os.path.expanduser(output_loc)
        if os.path.isdir(filename_fin):
            filename_fin = os.path.join(filename_fin, filename)
    else:
        filename_fin = filename

    return filename_fin


def write_xml(xml_str, output_loc=None, filename=None):
    """Outputs the XML content (string) into a file.

    If `output_loc` is supplied and it's a file (not directory), the output
    will be saved there and the `filename` is ignored.

    Args:
        xml_str: string with XML document
        output_loc: file or directory for saving the file
        filename: file name that will be used if `output_loc` is directory
            If it is needed and is not supplied, it will be generated
    """
    if not xml_str:
        raise Dump2PolarionException("No data to write.")
    filename_fin = _get_filename(output_loc=output_loc, filename=filename)

    with open(filename_fin, "w", encoding="utf-8") as xml_file:
        xml_file.write(get_unicode_str(xml_str))
    logger.info("Data written to '%s'", filename_fin)


def write_xml_root(xml_root, output_loc=None, filename=None):
    """Outputs the XML content (from XML element) into a file.

    If `output_loc` is supplied and it's a file (not directory), the output
    will be saved there and the `filename` is ignored.

    Args:
        xml_root: root element ot the XML document
        output_loc: file or directory for saving the file
        filename: file name that will be used if `output_loc` is directory
            If it is needed and is not supplied, it will be generated
    """
    if xml_root is None:
        raise Dump2PolarionException("No data to write.")
    filename_fin = _get_filename(output_loc=output_loc, filename=filename)

    et = etree.ElementTree(xml_root)
    et.write(filename_fin, xml_declaration=True, pretty_print=True, encoding="utf-8")
    logger.info("Data written to '%s'", filename_fin)


def get_xml_root(xml_file):
    """Returns XML root."""
    try:
        xml_root = etree.parse(os.path.expanduser(xml_file), NO_BLANKS_PARSER).getroot()
    # pylint: disable=broad-except
    except Exception as err:
        raise Dump2PolarionException("Failed to parse XML file '{}': {}".format(xml_file, err))
    return xml_root


def get_xml_root_from_str(xml_str):
    """Returns XML root from string."""
    try:
        xml_root = etree.fromstring(xml_str.encode("utf-8"), NO_BLANKS_PARSER)
    # pylint: disable=broad-except
    except Exception as err:
        raise Dump2PolarionException("Failed to parse XML string: {}".format(err))
    return xml_root


def etree_to_string(xml_root):
    """Returns string representation of element tree."""
    return get_unicode_str(etree.tostring(xml_root, encoding="utf-8"))


def prettify_xml(xml_root):
    """Returns pretty-printed string representation of element tree."""
    xml_string = etree.tostring(xml_root, encoding="utf-8", xml_declaration=True, pretty_print=True)
    return get_unicode_str(xml_string)


def get_session(credentials, config):
    """Gets requests session."""
    session = requests.Session()
    session.verify = False
    auth_url = config.get("auth_url")

    if auth_url:
        cookie = session.post(
            auth_url,
            data={
                "j_username": credentials[0],
                "j_password": credentials[1],
                "submit": "Log In",
                "rememberme": "true",
            },
            headers={"Content-Type": "application/x-www-form-urlencoded"},
        )
        if not cookie:
            raise Dump2PolarionException("Cookie was not retrieved from {}.".format(auth_url))
    else:
        # TODO: can be removed once basic auth is discontinued on prod
        session.auth = credentials

    return session


def get_testrun_id_config(config):
    """Gets testrun ID defined in config file."""
    config_testrun_id = config.get("xunit_import_properties") or {}
    return config_testrun_id.get("polarion-testrun-id")


def sorted_dict(dictionary):
    """Returns sorted dictionary."""
    return OrderedDict(sorted(dictionary.items()))



"""
Submit data to the Polarion Importer.
"""

import logging
import os

from dump2polarion import configuration, properties, utils
from dump2polarion.exceptions import Dump2PolarionException
from dump2polarion.verify import verify_submit

# pylint: disable=invalid-name
logger = logging.getLogger(__name__)


class SubmitResponse:
    """Response data from submit to Importer."""

    def __init__(self, response):
        self.response = response
        self.parsed_response = self.response2dict()
        self.job_ids = self.get_job_ids()

    def response2dict(self):
        """Returns dict of the response."""
        try:
            return self.response.json()
        # pylint: disable=broad-except
        except Exception:
            return None

    def get_job_ids(self):
        """Returns job IDs of the import."""
        if not self.parsed_response:
            return None
        try:
            job_ids = self.parsed_response["files"]["results.xml"]["job-ids"]
        except KeyError:
            return None
        if not job_ids or job_ids == [0]:
            return None
        return job_ids

    def get_error_message(self):
        """Returns job IDs of the import."""
        if self.parsed_response:
            return self.parsed_response["files"]["results.xml"].get("error-message")
        return None

    def validate_response(self):
        """Checks that the response is valid and import succeeded."""
        if self.response is None:
            logger.error("Failed to submit")
            return False

        if not self.response:
            logger.error(
                "HTTP status %d: failed to submit to %s",
                self.response.status_code,
                self.response.url,
            )
            return False

        if not self.parsed_response:
            logger.error("Submit to %s failed, invalid response received", self.response.url)
            return False

        error_message = self.get_error_message()
        if error_message:
            logger.error("Submit to %s failed with error", self.response.url)
            logger.debug("Error message: %s", error_message)
            return False

        if not self.job_ids:
            logger.error("Submit to %s failed to get job id", self.response.url)
            return False

        logger.info("Results received by the Importer (HTTP status %d)", self.response.status_code)
        logger.info("Job IDs: %s", self.job_ids)

        return True

    def __len__(self):
        return 1 if self.job_ids else 0

    def __repr__(self):
        return repr(self.response)


class SubmitConfig:
    """Configuration for data submit."""

    def __init__(self, xml_root, config, **kwargs):
        self.xml_root = xml_root
        self.config = config
        self.submit_kwargs = kwargs

        self.submit_target = None
        self.queue_url = None
        self.log_url = None
        self.credentials = None

        self.get_targets()
        self.get_credentials(**self.submit_kwargs)

    def get_targets(self):
        """Sets targets."""
        if self.xml_root.tag == "testcases":
            self.submit_target = self.config.get("testcase_taget")
            self.queue_url = self.config.get("testcase_queue")
            self.log_url = self.config.get("testcase_log")
        elif self.xml_root.tag == "testsuites":
            self.submit_target = self.config.get("xunit_target")
            self.queue_url = self.config.get("xunit_queue")
            self.log_url = self.config.get("xunit_log")
        elif self.xml_root.tag == "requirements":
            self.submit_target = self.config.get("requirement_target")
            self.queue_url = self.config.get("requirement_queue")
            self.log_url = self.config.get("requirement_log")
        else:
            raise Dump2PolarionException("Failed to submit to Polarion - submit target not found")

    def get_credentials(self, **kwargs):
        """Sets credentails."""
        login = (
            kwargs.get("user") or os.environ.get("POLARION_USERNAME") or self.config.get("username")
        )
        pwd = (
            kwargs.get("password")
            or os.environ.get("POLARION_PASSWORD")
            or self.config.get("password")
        )

        if not all([login, pwd]):
            raise Dump2PolarionException("Failed to submit to Polarion - missing credentials")

        self.credentials = (login, pwd)


def _get_xml_root(xml_root, xml_str, xml_file):
    if xml_root is not None:
        return xml_root
    if xml_str:
        return utils.get_xml_root_from_str(xml_str)
    if xml_file:
        return utils.get_xml_root(xml_file)
    raise Dump2PolarionException("Failed to submit to Polarion - no data supplied")


def submit(xml_root, submit_config, session, dry_run=None, **kwargs):
    """Submits data to the Polarion Importer."""
    properties.xunit_fill_testrun_id(xml_root, kwargs.get("testrun_id"))
    if dry_run is not None:
        properties.set_dry_run(xml_root, dry_run)
    xml_input = utils.etree_to_string(xml_root)

    logger.info("Submitting data to %s", submit_config.submit_target)
    files = {"file": ("results.xml", xml_input)}
    try:
        response = session.post(submit_config.submit_target, files=files)
    # pylint: disable=broad-except
    except Exception as err:
        logger.error(err)
        response = None

    return SubmitResponse(response)


# pylint: disable=too-many-arguments
def submit_and_verify(
    xml_str=None, xml_file=None, xml_root=None, config=None, session=None, dry_run=None, **kwargs
):
    """Submits data to the Polarion Importer and checks that it was imported."""
    try:
        config = config or configuration.get_config()
        xml_root = _get_xml_root(xml_root, xml_str, xml_file)
        submit_config = SubmitConfig(xml_root, config, **kwargs)
        session = session or utils.get_session(submit_config.credentials, config)
        submit_response = submit(xml_root, submit_config, session, dry_run=dry_run, **kwargs)
    except Dump2PolarionException as err:
        logger.error(err)
        return None

    valid_response = submit_response.validate_response()
    if not valid_response or kwargs.get("no_verify"):
        return submit_response.response

    response = verify_submit(
        session,
        submit_config.queue_url,
        submit_config.log_url,
        submit_response.job_ids,
        timeout=kwargs.get("verify_timeout"),
        log_file=kwargs.get("log_file"),
    )

    return response



"""
Functions for transforming results per Polarion project.

If the 'polarion-lookup-method' is set to 'custom', this is the place where you can
set the 'id' of the test case to desired value.
"""

import copy
import logging
import re

from dump2polarion.exporters import transform
from dump2polarion.exporters.verdicts import Verdicts

# pylint: disable=invalid-name
logger = logging.getLogger(__name__)


def set_cfme_caselevel(testcase, caselevels):
    """Converts tier to caselevel."""
    tier = testcase.get("caselevel")
    if tier is None:
        return

    try:
        caselevel = caselevels[int(tier)]
    except IndexError:
        # invalid value
        caselevel = "component"
    except ValueError:
        # there's already string value
        return

    testcase["caselevel"] = caselevel


def get_xunit_transform_cfme(config):
    """Return result transformation function for CFME."""
    skip_searches = [
        "SKIPME:",
        "Skipping due to these blockers",
        "BZ ?[0-9]+",
        "GH ?#?[0-9]+",
        "GH#ManageIQ",
    ]
    skips = re.compile("(" + ")|(".join(skip_searches) + ")")

    parametrize = config.get("cfme_parametrize", False)

    def results_transform(result):
        """Results transform for CFME."""
        verdict = result.get("verdict")
        if not verdict:
            return None

        result = copy.deepcopy(result)

        transform.setup_parametrization(result, parametrize)
        transform.include_class_in_title(result)
        transform.insert_source_info(result)

        verdict = verdict.strip().lower()
        # we want to submit PASS and WAIT results
        if verdict in Verdicts.PASS + Verdicts.WAIT:
            return result
        comment = result.get("comment")
        # ... and SKIP results where there is a good reason (blocker etc.)
        if verdict in Verdicts.SKIP and comment and skips.search(comment):
            # found reason for skip
            result["comment"] = comment.replace("SKIPME: ", "").replace("SKIPME", "")
            return result
        if verdict in Verdicts.FAIL and comment and "FAILME" in comment:
            result["comment"] = comment.replace("FAILME: ", "").replace("FAILME", "")
            return result
        # we don't want to report this result if here
        return None

    return results_transform


def get_testcases_transform_cfme(config):
    """Return test cases transformation function for CFME."""

    parametrize = config.get("cfme_parametrize", False)
    run_id = config.get("cfme_run_id")

    caselevels = config.get("docstrings") or {}
    caselevels = caselevels.get("valid_values") or {}
    caselevels = caselevels.get("caselevel") or []

    def testcase_transform(testcase):
        """Test cases transform for CFME."""
        testcase = copy.deepcopy(testcase)

        transform.setup_parametrization(testcase, parametrize)
        set_cfme_caselevel(testcase, caselevels)
        transform.preformat_plain_description(testcase)
        transform.add_unique_runid(testcase, run_id)
        transform.add_automation_link(testcase)

        return testcase

    return testcase_transform


# pylint: disable=unused-argument
def get_requirements_transform_cfme(config):
    """Return requirement transformation function for CFME."""

    def requirement_transform(requirement):
        """Requirements transform for CFME."""
        requirement = copy.deepcopy(requirement)

        if "id" in requirement:
            del requirement["id"]

        return requirement

    return requirement_transform


# pylint: disable=unused-argument
def get_xunit_transform_cmp(config):
    """Return result transformation function for CFME."""
    skip_searches = [
        "SKIPME:",
        "Skipping due to these blockers",
        "BZ ?[0-9]+",
        "GH ?#?[0-9]+",
        "GH#ManageIQ",
    ]
    skips = re.compile("(" + ")|(".join(skip_searches) + ")")

    def results_transform(result):
        """Results transform for CMP."""
        verdict = result.get("verdict")
        if not verdict:
            return None

        result = copy.deepcopy(result)

        # don't parametrize if not specifically configured
        if result.get("params"):
            del result["params"]

        classname = result.get("classname", "")
        if classname:
            # we don't need to pass classnames?
            del result["classname"]

        # if the "test_id" property is present, use it as test case ID
        test_id = result.get("test_id", "")
        if test_id:
            result["id"] = test_id

        verdict = verdict.strip().lower()
        # we want to submit PASS and WAIT results
        if verdict in Verdicts.PASS + Verdicts.WAIT:
            return result
        comment = result.get("comment")
        # ... and SKIP results where there is a good reason (blocker etc.)
        if verdict in Verdicts.SKIP and comment and skips.search(comment):
            # found reason for skip
            result["comment"] = comment.replace("SKIPME: ", "").replace("SKIPME", "")
            return result
        if verdict in Verdicts.FAIL and comment and "FAILME" in comment:
            result["comment"] = comment.replace("FAILME: ", "").replace("FAILME", "")
            return result
        # we don't want to report this result if here
        return None

    return results_transform


# pylint: disable=unused-argument
def get_requirements_transform_cloudtp(config):
    """Return requirement transformation function for CLOUDTP."""

    def requirement_transform(requirement):
        """Requirements transform for CLOUDTP."""
        requirement = copy.deepcopy(requirement)

        if "id" in requirement:
            del requirement["id"]
        # TODO: testing purposes, remove once ready
        if not requirement.get("assignee-id"):
            requirement["assignee-id"] = "mkourim"
        if not requirement.get("approver-ids"):
            requirement["approver-ids"] = "mkourim:approved"

        return requirement

    return requirement_transform


PROJECT_MAPPING_XUNIT = {
    "RHCF3": get_xunit_transform_cfme,
    "CMP": get_xunit_transform_cmp,
    "CLOUDTP": get_xunit_transform_cfme,
}

PROJECT_MAPPING_TESTCASES = {"RHCF3": get_testcases_transform_cfme}

PROJECT_MAPPING_REQ = {
    "RHCF3": get_requirements_transform_cfme,
    "CLOUDTP": get_requirements_transform_cloudtp,
}


def get_xunit_transform(config):
    """Returns results transformation function.

    The transformation function is returned by calling corresponding "getter" function.

    This allows customizations of results data according to requirements
    of the specific project.

    When no results data are returned, this result will be ignored
    and will not be written to the resulting XML.
    """

    project = config["polarion-project-id"]
    if project in PROJECT_MAPPING_XUNIT:
        return PROJECT_MAPPING_XUNIT[project](config)
    return None


def get_testcases_transform(config):
    """Returns test cases transformation function.

    The transformation function is returned by calling corresponding "getter" function.

    This allows customizations of test cases data according to requirements
    of the specific project.

    When no test cases data are returned, this test case will be ignored
    and will not be written to the resulting XML.
    """

    project = config["polarion-project-id"]
    if project in PROJECT_MAPPING_TESTCASES:
        return PROJECT_MAPPING_TESTCASES[project](config)
    return None


def get_requirements_transform(config):
    """Returns requirements transformation function.

    The transformation function is returned by calling corresponding "getter" function.

    This allows customizations of requirements data according to needs
    of the specific project.

    When no requirements data are returned, this requirement will be ignored
    and will not be written to the resulting XML.
    """

    project = config["polarion-project-id"]
    if project in PROJECT_MAPPING_REQ:
        return PROJECT_MAPPING_REQ[project](config)
    return None



# pylint: disable=missing-docstring,no-self-use,protected-access

import os

import pytest

from dump2polarion import configuration
from dump2polarion.exceptions import Dump2PolarionException
from tests import conf


class TestConfiguration:
    def test_nonexistant(self):
        with pytest.raises(Dump2PolarionException) as excinfo:
            configuration.get_config("nonexistant")
        assert "Cannot open config file nonexistant" in str(excinfo.value)

    def test_default_w_project(self):
        with pytest.raises(Dump2PolarionException) as excinfo:
            configuration.get_config()
        assert "Failed to find configuration file" in str(excinfo.value)

    def test_default_no_project(self):
        with pytest.raises(Dump2PolarionException) as excinfo:
            configuration.get_config(load_project_conf=False)
        assert "No configuration file or values passed" in str(excinfo.value)

    def test_keys_missing(self):
        project_id = {"polarion-project-id": "RHCF3"}
        with pytest.raises(Dump2PolarionException) as excinfo:
            configuration.get_config(config_values=project_id)
        assert "Failed to find following keys in config file" in str(excinfo.value)

    def test_user(self, config_e2e):
        cfg = configuration.get_config(config_e2e)
        assert cfg["xunit_import_properties"]["polarion-dry-run"] is False
        assert cfg["username"] == "user1"
        assert cfg["polarion-project-id"] == "RHCF3"

    @pytest.mark.parametrize("config", ["polarion_tools_legacy.yaml", "polarion_tools.yaml"])
    def test_populate(self, config):
        conf_file = os.path.join(conf.DATA_PATH, config)
        cfg = configuration.get_config(conf_file)
        polarion_url = cfg.get("polarion_url")
        for key in configuration.URLS:
            assert key in cfg
            if polarion_url and cfg.get(key):
                assert polarion_url in cfg[key]

    def test_check_config(self):
        cfg = {}
        with pytest.raises(Dump2PolarionException) as excinfo:
            configuration._check_config(cfg)
        assert "Failed to find following keys in config file" in str(excinfo.value)



"""
Helper functions for handling JSON data from Ostriz.
"""

import datetime
import json
import logging
import os
from collections import OrderedDict

import requests
from packaging.version import InvalidVersion, Version

from dump2polarion.exceptions import Dump2PolarionException, NothingToDoException
from dump2polarion.exporters import xunit_exporter

# pylint: disable=invalid-name
logger = logging.getLogger(__name__)


IGNORED_PARAMS = {"browserVersion", "browserPlatform", "browserName"}


def _get_json(location):
    """Reads JSON data from file or URL."""
    location = os.path.expanduser(location)
    try:
        if os.path.isfile(location):
            with open(location, encoding="utf-8") as json_data:
                return json.load(json_data, object_pairs_hook=OrderedDict).get("tests")
        elif "http" in location:
            json_data = requests.get(location)
            if not json_data:
                raise Dump2PolarionException("Failed to download")
            return json.loads(json_data.text, object_pairs_hook=OrderedDict).get("tests")
        else:
            raise Dump2PolarionException("Invalid location")
    except Exception as err:
        raise Dump2PolarionException("Failed to parse JSON from {}: {}".format(location, err))


def _get_testrun_id(version):
    """Gets testrun id out of the version string.

    Remove trailing version dates / hashes separated by - / _
    Then use packaging Version to parse the version string
    Then cast each release component to string for joining with _

    Build and returns testrun ID that looks like x_y_z_a from version x.y.z.a-20181114_abcdef
    """
    try:
        v = Version(version.split("-")[0].split("_")[0])
        build_base = "_".join([str(i) for i in v.release])
    except InvalidVersion:
        raise Dump2PolarionException("InvalidVersion parsing testrun ID from {}".format(version))
    except Exception:
        # not in expected format
        raise Dump2PolarionException("Exception parsing testrun ID from {}".format(version))
    return build_base


def _calculate_duration(start_time, finish_time):
    """Calculates how long it took to execute the testcase."""
    if not (start_time and finish_time):
        return 0
    start = datetime.datetime.fromtimestamp(start_time)
    finish = datetime.datetime.fromtimestamp(finish_time)
    duration = finish - start

    decimals = float("0." + str(duration.microseconds))
    return duration.seconds + decimals


def _get_testname(test_path):
    """Gets test name out of full test path."""
    path_end = test_path.find(".py/")
    if path_end:
        offset = path_end + 4
        return test_path[offset:]
    return None


def _filter_parameters(parameters):
    """Filters the ignored parameters out."""
    if not parameters:
        return None
    return OrderedDict(
        (param, value) for param, value in parameters.items() if param not in IGNORED_PARAMS
    )


def _append_record(test_data, results, test_path):
    """Adds data of single testcase results to results database."""
    statuses = test_data.get("statuses")
    jenkins_data = test_data.get("jenkins") or {}

    data = [
        ("title", test_data.get("test_name") or _get_testname(test_path)),
        ("verdict", statuses.get("overall")),
        ("source", test_data.get("source")),
        ("job_name", jenkins_data.get("job_name")),
        ("run", jenkins_data.get("build_number")),
        ("params", _filter_parameters(test_data.get("params"))),
        (
            "time",
            _calculate_duration(test_data.get("start_time"), test_data.get("finish_time")) or 0,
        ),
    ]
    test_id = test_data.get("polarion")
    if test_id:
        if isinstance(test_id, list):
            test_id = test_id[0]
        data.append(("test_id", test_id))

    results.append(OrderedDict(data))


def _comp_finish_time(test_data, last_finish_time):
    curr_finish_time = test_data.get("finish_time") or 0
    if curr_finish_time > last_finish_time[0]:
        last_finish_time[0] = curr_finish_time


def _parse_ostriz(ostriz_data):
    """Reads the content of the input JSON and returns testcases results."""
    if not ostriz_data:
        raise NothingToDoException("No data to import")

    results = []
    found_build = None
    last_finish_time = [0]
    for test_path, test_data in ostriz_data.items():
        curr_build = test_data.get("build")
        if not curr_build:
            continue

        # set `found_build` from first record where it's present
        if not found_build:
            found_build = curr_build

        # make sure we are collecting data for the same build
        if found_build != curr_build:
            continue

        if not test_data.get("statuses"):
            continue

        _append_record(test_data, results, test_path)
        _comp_finish_time(test_data, last_finish_time)

    if last_finish_time[0]:
        logger.info("Last result finished at %s", last_finish_time[0])

    testrun_id = _get_testrun_id(found_build)
    return xunit_exporter.ImportedData(results=results, testrun=testrun_id)


# pylint: disable=unused-argument
def import_ostriz(location, **kwargs):
    """Reads Ostriz's data and returns imported data."""
    ostriz_data = _get_json(location)
    return _parse_ostriz(ostriz_data)



# pylint: disable=missing-docstring,redefined-outer-name,no-self-use,protected-access

import os
from io import StringIO

import pytest

from dump2polarion.exceptions import Dump2PolarionException
from dump2polarion.results import jsontools
from tests import conf


class TestJSONImport:
    def test_import_orig_data(self):
        json_file = os.path.join(conf.DATA_PATH, "test_run_import.json")
        data = jsontools.import_json(json_file)
        assert hasattr(data, "results")
        assert len(data.results) == 18
        passed_num = failed_num = skipped_num = 0
        for result in data.results:
            if result["verdict"] == "passed":
                passed_num += 1
            elif result["verdict"] == "failed":
                assert result["comment"]
                failed_num += 1
            elif result["verdict"] == "skipped":
                assert result["comment"]
                skipped_num += 1
        assert passed_num == 9
        assert failed_num == 4
        assert skipped_num == 3
        assert hasattr(data, "testrun")
        assert data.testrun is None

    def test_invalid_input(self):
        invalid_content = "foo"
        input_file = StringIO(invalid_content)
        with pytest.raises(Dump2PolarionException) as excinfo:
            jsontools.import_json(input_file)
        input_file.close()
        assert "Cannot load results from" in str(excinfo.value)



# pylint: disable=missing-docstring,no-self-use,unused-argument,invalid-name

import os

from dump2polarion import verify

SEARCH_QUEUE = {
    "jobsPerPage": 50,
    "maxPages": 2,
    "currentPage": 1,
    "jobs": [
        {"id": 17977, "status": "SUCCESS", "logstashURL": "http://logstash/00017977.log"},
        {"id": 17974, "status": "FAILED", "logstashURL": "http://logstash/00017974.log"},
        {"id": 17976, "status": "SUCCESS", "logstashURL": "http://logstash/00017976.log"},
    ],
}


class DummyResponse:
    def __init__(self, response):
        self.status_code = 200
        self.response = response

    def __len__(self):
        return 1

    def json(self):
        return self.response

    @property
    def content(self):
        return self.response


class DummySession:
    def __init__(self, get):
        self._get = get

    def get(self, *args, **kwargs):
        return self._get()


def download_queue_data(*args, **kwargs):
    return SEARCH_QUEUE


# pylint: disable=too-many-public-methods
class TestQueueSearch:

    # object init
    def test_session_missing(self, captured_log):
        vq = verify.get_queue_obj(None, "foo", None)
        assert "Missing requests session, skipping" in captured_log.getvalue()
        assert vq.skip is True

    def test_url_missing(self, captured_log):
        vq = verify.get_queue_obj("foo", None, None)
        assert "The queue url is not configured, skipping" in captured_log.getvalue()
        assert vq.skip is True

    def test_init(self):
        vq = verify.get_queue_obj("foo", "bar", None)
        assert vq.skip is False

    # download queue
    def test_download_queue_none(self):
        vq = verify.get_queue_obj(DummySession(lambda: DummyResponse(None)), "bar", None)
        response = vq.download_queue([1, 2])
        assert response is None

    def test_download_queue_data(self):
        vq = verify.get_queue_obj(DummySession(lambda: DummyResponse(SEARCH_QUEUE)), "bar", None)
        response = vq.download_queue([1, 2])
        assert response == SEARCH_QUEUE

    def test_download_queue_failed(self):
        vq = verify.get_queue_obj(DummySession(lambda: False), "bar", None)
        response = vq.download_queue([1, 2])
        assert response is None

    def test_download_queue_exception(self, captured_log):
        def _raise():
            raise Exception("TestFail")

        vq = verify.get_queue_obj(DummySession(_raise), "bar", None)
        response = vq.download_queue([1, 2])
        assert response is None
        assert "TestFail" in captured_log.getvalue()

    # find job in queue
    def test_job_not_found(self):
        vq = verify.get_queue_obj("foo", "bar", None)
        vq.download_queue = download_queue_data
        outcome = vq.find_jobs([17978, 17975])
        assert outcome == []

    def test_job_found(self):
        vq = verify.get_queue_obj("foo", "bar", None)
        vq.download_queue = download_queue_data
        outcome = vq.find_jobs([17977, 17976])
        assert outcome[0] == SEARCH_QUEUE["jobs"][0]
        assert outcome[1] == SEARCH_QUEUE["jobs"][2]

    # job log handling
    def test_get_log_failed(self, tmpdir, captured_log):
        log_file = os.path.join(str(tmpdir), "out.log")
        jobs = [{"id": "111"}]
        vq = verify.get_queue_obj(
            DummySession(lambda: DummyResponse(None)), "bar", "http://example.com"
        )
        vq.get_logs(jobs, log_file)
        assert not os.path.exists(log_file)
        assert "Failed to download log file" in captured_log.getvalue()

    def test_get_log_saved(self, tmpdir):
        log_file = os.path.join(str(tmpdir), "out.log")
        jobs = [{"id": "111"}]
        vq = verify.get_queue_obj(
            DummySession(lambda: DummyResponse(b"content")), "bar", "http://example.com"
        )
        vq.get_logs(jobs, log_file)
        assert os.path.exists(log_file)

    def test_get_log_displayed(self, captured_log):
        jobs = [{"id": "111"}]
        vq = verify.get_queue_obj(
            DummySession(lambda: DummyResponse(None)), "bar", "http://example.com"
        )
        vq.get_logs(jobs)
        assert "Submit log for job 111: http://example.com?jobId=111" in captured_log.getvalue()

    def test_get_log_exception(self, tmpdir, captured_log):
        def _raise():
            raise Exception("TestFail")

        log_file = os.path.join(str(tmpdir), "out.log")
        jobs = [{"id": "111"}]
        vq = verify.get_queue_obj(DummySession(_raise), "bar", "http://example.com")
        vq.get_logs(jobs, log_file)
        assert not os.path.exists(log_file)
        assert "TestFail" in captured_log.getvalue()

    # verify submit
    def test_queue_empty_queue(self, captured_log):
        vq = verify.get_queue_obj("foo", "bar", None)
        vq.download_queue = lambda *args: None
        outcome = vq.verify_submit([17974], timeout=0.0000001, delay=0.0000001)
        assert outcome is False
        assert "not updated" in captured_log.getvalue()

    def test_queue_submit_failed_one(self, captured_log):
        vq = verify.get_queue_obj("foo", "bar", None)
        vq.download_queue = download_queue_data
        outcome = vq.verify_submit([17974, 17976, 17977], timeout=0.0000001, delay=0.0000001)
        assert outcome is False
        assert "status: FAILED" in captured_log.getvalue()
        assert "Some import jobs failed" in captured_log.getvalue()

    def test_queue_submit_failed_all(self, captured_log):
        vq = verify.get_queue_obj("foo", "bar", None)
        vq.download_queue = download_queue_data
        outcome = vq.verify_submit([17974], timeout=0.0000001, delay=0.0000001)
        assert outcome is False
        assert "status: FAILED" in captured_log.getvalue()
        assert "Import failed!" in captured_log.getvalue()

    def test_queue_submit_not_found_one(self, captured_log):
        vq = verify.get_queue_obj("foo", "bar", None)
        vq.download_queue = download_queue_data
        outcome = vq.verify_submit([17976, 17978], timeout=0.0000001, delay=0.0000001)
        assert outcome is False
        assert "not updated" in captured_log.getvalue()
        assert "Import failed!" in captured_log.getvalue()

    def test_queue_submit_not_found_all(self, captured_log):
        vq = verify.get_queue_obj("foo", "bar", None)
        vq.download_queue = download_queue_data
        outcome = vq.verify_submit([17978], timeout=0.0000001, delay=0.0000001)
        assert outcome is False
        assert "not updated" in captured_log.getvalue()
        assert "Import failed!" in captured_log.getvalue()

    def test_queue_submit_found(self, captured_log):
        vq = verify.get_queue_obj("foo", "bar", None)
        vq.download_queue = download_queue_data
        outcome = vq.verify_submit([17976, 17977], timeout=0.0000001, delay=0.0000001)
        assert outcome
        assert "successfully updated" in captured_log.getvalue()

    def test_queue_submit_skip(self):
        vq = verify.get_queue_obj("foo", "bar", None)
        vq.skip = True
        outcome = vq.verify_submit([17977], timeout=0.0000001, delay=0.0000001)
        assert outcome is False



# pylint: disable=missing-docstring,redefined-outer-name,no-self-use

import os
import shutil

import pytest
from mock import patch

from dump2polarion import dumper_cli
from dump2polarion.exceptions import Dump2PolarionException
from dump2polarion.exporters.transform import only_passed_and_wait
from dump2polarion.results import dbtools
from tests import conf


class TestDumperCLIUnits:
    def test_get_args(self):
        args = dumper_cli.get_args(["-i", "dummy", "-t", "testrun_id"])
        assert args.input_file == "dummy"
        assert args.output_file is None
        assert args.testrun_id == "testrun_id"
        assert args.config_file is None
        assert args.no_submit is False
        assert args.user is None
        assert args.password is None
        assert args.force is False
        assert args.log_level is None

    def test_testrun_id_match(self):
        args = dumper_cli.get_args(["-i", "dummy", "-t", "5_8_0_17"])
        found = dumper_cli.get_testrun_id(args, {}, "5_8_0_17")
        assert found == "5_8_0_17"

    def test_testrun_id_nomatch(self):
        args = dumper_cli.get_args(["-i", "dummy", "-t", "5_8_0_17"])
        with pytest.raises(Dump2PolarionException) as excinfo:
            dumper_cli.get_testrun_id(args, {}, "5_8_0_18")
        assert "differ" in str(excinfo.value)

    def test_testrun_id_config_nomatch(self):
        args = dumper_cli.get_args(["-i", "dummy", "-t", "5_8_0_18"])
        with pytest.raises(Dump2PolarionException) as excinfo:
            dumper_cli.get_testrun_id(
                args, {"xunit_import_properties": {"polarion-testrun-id": "5_8_0_17"}}, "5_8_0_18"
            )
        assert "differ" in str(excinfo.value)

    def test_testrun_id_force(self):
        args = dumper_cli.get_args(["-i", "dummy", "-t", "5_8_0_18", "--force"])
        found = dumper_cli.get_testrun_id(args, {}, "5_8_0_17")
        assert found == "5_8_0_18"

    def test_testrun_id_missing(self):
        args = dumper_cli.get_args(["-i", "dummy"])
        with pytest.raises(Dump2PolarionException) as excinfo:
            dumper_cli.get_testrun_id(args, {}, None)
        assert "The testrun id was not specified" in str(excinfo.value)

    def test_submit_if_ready_noxml(self, config_prop):
        args = dumper_cli.get_args(["-i", "submit.txt"])
        submit_args = dumper_cli.get_submit_args(args)
        with patch("dump2polarion.submit_and_verify", return_value=True):
            retval = dumper_cli.submit_if_ready(args, submit_args, config_prop)
        assert retval is None

    def test_submit_if_ready_nosubmit(self, tmpdir, config_prop):
        xml_content = "<testcases foo=bar>"
        xml_file = tmpdir.join("submit_nosubmit.xml")
        xml_file.write(xml_content)
        args = dumper_cli.get_args(["-i", str(xml_file), "--no-submit"])
        submit_args = dumper_cli.get_submit_args(args)
        with patch("dump2polarion.submit_and_verify", return_value=True):
            retval = dumper_cli.submit_if_ready(args, submit_args, config_prop)
        assert retval == 0

    def test_submit_if_ready_failed(self, tmpdir, config_prop):
        xml_content = "<testcases foo=bar>"
        xml_file = tmpdir.join("submit_failed.xml")
        xml_file.write(xml_content)
        args = dumper_cli.get_args(["-i", str(xml_file)])
        submit_args = dumper_cli.get_submit_args(args)
        with patch("dump2polarion.submit_and_verify", return_value=False):
            retval = dumper_cli.submit_if_ready(args, submit_args, config_prop)
        assert retval == 2

    @pytest.mark.parametrize("tag", ("testsuites", "testcases", "requirements"))
    def test_submit_if_ready_ok(self, tmpdir, config_prop, tag):
        xml_content = "<{} foo=bar><properties>".format(tag)
        xml_file = tmpdir.join("submit_ready.xml")
        xml_file.write(xml_content)
        args = dumper_cli.get_args(["-i", str(xml_file)])
        submit_args = dumper_cli.get_submit_args(args)
        with patch("dump2polarion.submit_and_verify", return_value=True):
            retval = dumper_cli.submit_if_ready(args, submit_args, config_prop)
        assert retval == 0


E2E_DATA = [
    # default transform
    ("junit-report.xml", "junit_transform.xml", ["-t", "5_8_0_17"], None),
    ("workitems_ids.csv", "complete_transform.xml", [], None),
    ("workitems_ids.sqlite3", "complete_transform.xml", [], None),
    ("ostriz.json", "ostriz_transform.xml", [], None),
    # no transform
    ("junit-report.xml", "junit_notransform.xml", ["-t", "5_8_0_17"], lambda arg: arg),
    ("workitems_ids.csv", "complete_notransform.xml", [], lambda arg: arg),
    ("workitems_ids.sqlite3", "complete_notransform.xml", [], lambda arg: arg),
    ("ostriz.json", "ostriz_notransform.xml", [], lambda arg: arg),
    # only passed and wait
    (
        "junit-report.xml",
        "junit_passed_wait_transform.xml",
        ["-t", "5_8_0_17"],
        only_passed_and_wait,
    ),
    ("workitems_ids.csv", "complete_passed_wait_transform.xml", [], only_passed_and_wait),
    ("workitems_ids.sqlite3", "complete_passed_wait_transform.xml", [], only_passed_and_wait),
    ("ostriz.json", "ostriz_passed_wait_transform.xml", [], only_passed_and_wait),
]


class TestDumperCLIE2E:
    @pytest.mark.parametrize("data", E2E_DATA, ids=[d[0] for d in E2E_DATA])
    @pytest.mark.parametrize("submit", (True, False), ids=("submit", "nosubmit"))
    # pylint: disable=too-many-locals
    def test_main_formats(self, tmpdir, config_e2e, data, submit):
        input_name, golden_output, extra_args, transform_func = data
        # copy the sqlite db so the records are not marked as exported
        if "sqlite3" in input_name:
            orig_db_file = os.path.join(conf.DATA_PATH, input_name)
            db_file = os.path.join(str(tmpdir), "workitems_copy.sqlite3")
            shutil.copy(orig_db_file, db_file)
            input_file = db_file
        else:
            input_file = os.path.join(conf.DATA_PATH, input_name)
        output_file = tmpdir.join("out.xml")
        args = ["-i", input_file, "-o", str(output_file), "-c", config_e2e]
        args.extend(extra_args)
        if not submit:
            args.append("-n")

        with patch("dump2polarion.submit_and_verify", return_value=True), patch(
            "dump2polarion.dumper_cli.utils.init_log"
        ):
            retval = dumper_cli.main(args, transform_func=transform_func)
        assert retval == 0

        with open(os.path.join(conf.DATA_PATH, golden_output), encoding="utf-8") as golden_xml:
            parsed = golden_xml.read()
        with open(str(output_file), encoding="utf-8") as out_xml:
            produced = out_xml.read()
        assert produced == parsed

    def test_main_missing_testrun(self, tmpdir, config_e2e, captured_log):
        input_file = os.path.join(conf.DATA_PATH, "junit-report.xml")
        output_file = tmpdir.join("out.xml")
        args = ["-i", input_file, "-o", str(output_file), "-c", config_e2e, "-n"]

        with patch("dump2polarion.submit_and_verify", return_value=True):
            retval = dumper_cli.main(args)
        assert retval == 1
        assert "The testrun id was not specified" in captured_log.getvalue()

        with pytest.raises(IOError) as excinfo:
            open(str(output_file))
        assert "No such file or directory" in str(excinfo.value)

    def test_main_submit_ready(self, config_e2e):
        input_file = os.path.join(conf.DATA_PATH, "complete_transform.xml")
        args = ["-i", input_file, "-c", config_e2e]

        with patch("dump2polarion.submit_and_verify", return_value=True), patch(
            "dump2polarion.dumper_cli.utils.init_log"
        ):
            retval = dumper_cli.main(args)
        assert retval == 0

    def test_main_noreport(self, config_e2e):
        input_file = os.path.join(conf.DATA_PATH, "noreport.csv")
        args = ["-i", input_file, "-c", config_e2e]

        with patch("dump2polarion.submit_and_verify", return_value=True), patch(
            "dump2polarion.dumper_cli.utils.init_log"
        ):
            retval = dumper_cli.main(args)
        assert retval == 0

    def test_main_noresults(self, config_e2e, captured_log):
        input_file = os.path.join(conf.DATA_PATH, "noresults.csv")
        args = ["-i", input_file, "-c", config_e2e]

        with patch("dump2polarion.submit_and_verify", return_value=True):
            retval = dumper_cli.main(args)
        assert retval == 1
        assert "No results read from" in captured_log.getvalue()

    def test_main_unconfigured(self, captured_log):
        input_file = os.path.join(conf.DATA_PATH, "noreport.csv")
        args = ["-i", input_file]

        with patch("dump2polarion.submit_and_verify", return_value=True):
            retval = dumper_cli.main(args)
        assert retval == 1
        assert "Failed to find configuration file" in captured_log.getvalue()

    def test_main_noconfig(self, captured_log):
        input_file = os.path.join(conf.DATA_PATH, "noreport.csv")
        args = ["-i", input_file, "-c", "nonexistent"]

        with patch("dump2polarion.submit_and_verify", return_value=True):
            retval = dumper_cli.main(args)
        assert retval == 1
        assert "Cannot open config file" in captured_log.getvalue()

    def test_main_submit_failed(self, tmpdir, config_e2e):
        input_file = os.path.join(conf.DATA_PATH, "workitems_ids.csv")
        output_file = tmpdir.join("out.xml")
        args = ["-i", input_file, "-o", str(output_file), "-c", config_e2e]

        with patch("dump2polarion.submit_and_verify", return_value=False), patch(
            "dump2polarion.dumper_cli.utils.init_log"
        ):
            retval = dumper_cli.main(args)
        assert retval == 2

        golden_output = "complete_transform.xml"
        with open(os.path.join(conf.DATA_PATH, golden_output), encoding="utf-8") as golden_xml:
            parsed = golden_xml.read()
        with open(str(output_file), encoding="utf-8") as out_xml:
            produced = out_xml.read()
        assert produced == parsed

    # pylint: disable=too-many-locals,protected-access
    def test_main_submit_db(self, tmpdir, config_e2e):
        orig_db_file = os.path.join(conf.DATA_PATH, "workitems_ids.sqlite3")
        db_file = os.path.join(str(tmpdir), "workitems_copy.sqlite3")
        shutil.copy(orig_db_file, db_file)

        output_file = tmpdir.join("out.xml")
        args = ["-i", db_file, "-o", str(output_file), "-c", config_e2e]

        with patch("dump2polarion.submit_and_verify", return_value=True), patch(
            "dump2polarion.dumper_cli.utils.init_log"
        ):
            retval = dumper_cli.main(args)
        assert retval == 0

        golden_output = "complete_transform.xml"
        with open(os.path.join(conf.DATA_PATH, golden_output), encoding="utf-8") as golden_xml:
            parsed = golden_xml.read()
        with open(str(output_file), encoding="utf-8") as out_xml:
            produced = out_xml.read()
        assert produced == parsed

        conn = dbtools._open_sqlite(db_file)
        cur = conn.cursor()
        select = "SELECT count(*) FROM testcases WHERE exported == 'yes'"
        cur.execute(select)
        num = cur.fetchone()
        conn.close()
        assert num[0] == 13



# pylint: disable=missing-docstring,redefined-outer-name,no-self-use,protected-access

import copy
import os

import pytest
from lxml import etree

from dump2polarion.exceptions import Dump2PolarionException, NothingToDoException
from dump2polarion.exporters.xunit_exporter import ImportedData, XunitExport
from dump2polarion.results.importer import import_results
from dump2polarion.utils import get_unicode_str
from tests import conf


@pytest.fixture(scope="module")
def records_ids():
    csv_file = os.path.join(conf.DATA_PATH, "workitems_ids.csv")
    return import_results(csv_file)


@pytest.fixture(scope="module")
def records_names():
    csv_file = os.path.join(conf.DATA_PATH, "workitems_ids.csv")
    records = import_results(csv_file)
    for res in records.results:
        res.pop("id")
    return records


def test_top_element(config_prop, records_ids):
    exporter = XunitExport("5_8_0_17", records_ids, config_prop, transform_func=lambda: None)
    top_element = exporter._top_element()
    parsed = "<testsuites><!--Generated for testrun 5_8_0_17--></testsuites>".strip()
    top_element_str = get_unicode_str(etree.tostring(top_element, encoding="utf-8").strip())
    assert top_element_str == parsed


class TestConfigPropMixin:
    @pytest.fixture(autouse=True)
    def base_config_prop(self, config_prop):
        self.config_prop = config_prop


class TestProperties(TestConfigPropMixin):
    def test_properties_element(self, records_ids):
        exporter = XunitExport(
            "5_8_0_17", records_ids, self.config_prop, transform_func=lambda: None
        )
        top_element = exporter._top_element()
        properties_element = exporter._properties_element(top_element)
        parsed = (
            "<properties>"
            '<property name="polarion-testrun-id" value="5_8_0_17"/>'
            '<property name="polarion-project-id" value="RHCF3"/>'
            '<property name="polarion-dry-run" value="False"/>'
            '<property name="polarion-response-test" value="test"/>'
            '<property name="polarion-testrun-status-id" value="inprogress"/>'
            "</properties>".strip()
        )
        properties_str = get_unicode_str(
            etree.tostring(properties_element, encoding="utf-8").strip()
        )
        assert properties_str == parsed

    def test_properties_lookup_config(self, records_names):
        new_config = copy.deepcopy(self.config_prop)
        new_config["xunit_import_properties"]["polarion-lookup-method"] = "id"
        exporter = XunitExport("5_8_0_17", records_names, new_config, transform_func=lambda: None)
        top_element = exporter._top_element()
        properties_element = exporter._properties_element(top_element)
        exporter._fill_lookup_prop(properties_element)
        properties_str = get_unicode_str(
            etree.tostring(properties_element, encoding="utf-8").strip()
        )
        assert '<property name="polarion-lookup-method" value="id"/>' in properties_str

    def test_properties_invalid_lookup(self, records_ids):
        new_config = copy.deepcopy(self.config_prop)
        new_config["xunit_import_properties"]["polarion-lookup-method"] = "invalid"
        exporter = XunitExport("5_8_0_17", records_ids, new_config)
        with pytest.raises(Dump2PolarionException) as excinfo:
            exporter.export()
        assert "Invalid value 'invalid' for the 'polarion-lookup-method'" in str(excinfo.value)


class TestE2E(TestConfigPropMixin):
    def test_e2e_noresults(self, records_ids):
        exporter = XunitExport(
            "5_8_0_17", records_ids, self.config_prop, transform_func=lambda arg: None
        )
        with pytest.raises(NothingToDoException) as excinfo:
            exporter.export()
        assert "Nothing to export" in str(excinfo.value)

    def test_e2e_missing_results(self):
        new_records = ImportedData(results=[], testrun=None)
        exporter = XunitExport(
            "5_8_0_17", new_records, self.config_prop, transform_func=lambda arg: None
        )
        with pytest.raises(NothingToDoException) as excinfo:
            exporter._fill_tests_results(None)
        assert "Nothing to export" in str(excinfo.value)

    def test_e2e_ids_notransform(self, records_ids):
        exporter = XunitExport(
            "5_8_0_17", records_ids, self.config_prop, transform_func=lambda arg: arg
        )
        complete = exporter.export()
        fname = "complete_notransform.xml"
        with open(os.path.join(conf.DATA_PATH, fname), encoding="utf-8") as input_xml:
            parsed = input_xml.read()
        assert complete == parsed

    def test_e2e_ids_transform(self, records_ids):
        exporter = XunitExport("5_8_0_17", records_ids, self.config_prop)
        complete = exporter.export()
        fname = "complete_transform.xml"
        with open(os.path.join(conf.DATA_PATH, fname), encoding="utf-8") as input_xml:
            parsed = input_xml.read()
        assert complete == parsed

    def test_e2e_names_notransform(self, records_names):
        exporter = XunitExport(
            "5_8_0_17", records_names, self.config_prop, transform_func=lambda arg: arg
        )
        complete = exporter.export()
        fname = "complete_notransform_name.xml"
        with open(os.path.join(conf.DATA_PATH, fname), encoding="utf-8") as input_xml:
            parsed = input_xml.read()
        assert complete == parsed

    def test_e2e_names_transform(self, records_names):
        exporter = XunitExport("5_8_0_17", records_names, self.config_prop)
        complete = exporter.export()
        fname = "complete_transform_name.xml"
        with open(os.path.join(conf.DATA_PATH, fname), encoding="utf-8") as input_xml:
            parsed = input_xml.read()
        assert complete == parsed



# pylint: disable=missing-docstring,redefined-outer-name,no-self-use,protected-access,invalid-name

import pytest

from dump2polarion.exporters import transform_projects

# format:
# original record, expected record, parameter id

NOT_PASSED = [
    ({"title": "test_1", "verdict": "failed"}, None, "failed_removed"),
    ({"title": "test_1", "verdict": "skipped"}, None, "skipped_nobz"),
    (
        {"title": "test_1", "verdict": "skipped", "comment": "BZ123"},
        {"title": "test_1", "verdict": "skipped", "comment": "BZ123"},
        "skipped_bz",
    ),
    (
        {"title": "test_1", "verdict": "skipped", "comment": "BZ 123"},
        {"title": "test_1", "verdict": "skipped", "comment": "BZ 123"},
        "skipped_bz",
    ),
    (
        {"title": "test_1", "verdict": "skipped", "comment": "SKIPME: foo"},
        {"title": "test_1", "verdict": "skipped", "comment": "foo"},
        "skipped_skipme",
    ),
    (
        {"title": "test_1", "verdict": "failed", "comment": "FAILME: foo"},
        {"title": "test_1", "verdict": "failed", "comment": "foo"},
        "failed_failme",
    ),
    ({"title": "test_1", "verdict": "wait"}, {"title": "test_1", "verdict": "wait"}, "waiting"),
]

RHCF3_ONLY = [
    (
        {
            "classname": "cfme.tests.rest.TestRESTAPI",
            "title": "test_1",
            "verdict": "passed",
            "comment": "comment",
            "file": "cfme/tests/rest.py",
        },
        {
            "title": "TestRESTAPI.test_1",
            "verdict": "passed",
            "comment": "comment",
            "file": "cfme/tests/rest.py",
        },
        "append_class",
    ),
    (
        {
            "classname": "TestRESTAPI",
            "title": "test_1",
            "verdict": "passed",
            "comment": "comment",
            "file": "cfme/tests/rest.py",
        },
        {
            "title": "test_1",
            "verdict": "passed",
            "comment": "comment",
            "file": "cfme/tests/rest.py",
        },
        "no_append_class",
    ),
    (
        {
            "classname": "cfme.tests.rest",
            "title": "test_1",
            "verdict": "passed",
            "comment": "comment",
            "file": "cfme/tests/rest.py",
        },
        {
            "title": "test_1",
            "verdict": "passed",
            "comment": "comment",
            "file": "cfme/tests/rest.py",
        },
        "no_append_class",
    ),
    (
        {
            "title": "test_1",
            "verdict": "passed",
            "source": "jenkins",
            "job_name": "downstream",
            "run": "123",
        },
        {
            "title": "test_1",
            "verdict": "passed",
            "comment": "Source: jenkins/downstream/123",
            "source": "jenkins",
            "job_name": "downstream",
            "run": "123",
        },
        "passed_comment",
    ),
]

CMP_ONLY = [
    (
        {"classname": "test_1", "title": "test_1", "verdict": "passed", "test_id": "CMP-9985"},
        {"title": "test_1", "verdict": "passed", "test_id": "CMP-9985", "id": "CMP-9985"},
        "add_id",
    )
]

RHCF3_DATA = RHCF3_ONLY + NOT_PASSED
CMP_DATA = CMP_ONLY + NOT_PASSED


class TestTransform:
    @pytest.fixture(scope="class")
    def config_rhcf3(self):
        return {"polarion-project-id": "RHCF3"}

    @pytest.fixture(scope="class")
    def config_cmp(self):
        return {"polarion-project-id": "CMP"}

    @pytest.mark.parametrize("data", RHCF3_DATA, ids=[d[2] for d in RHCF3_DATA])
    def test_transform_rhcf3(self, config_rhcf3, data):
        tfunc = transform_projects.get_xunit_transform(config_rhcf3)
        result = tfunc(data[0])
        assert result == data[1]

    @pytest.mark.parametrize("data", CMP_DATA, ids=[d[2] for d in CMP_DATA])
    def test_transform_cmp(self, config_cmp, data):
        tfunc = transform_projects.get_xunit_transform(config_cmp)
        result = tfunc(data[0])
        assert result == data[1]



# pylint: disable=missing-docstring,no-self-use,redefined-outer-name

import copy
import os

import pytest

from dump2polarion.exporters.xunit_exporter import XunitExport
from dump2polarion.results.importer import import_results
from tests import conf


@pytest.fixture(scope="module")
def records_names():
    input_file = os.path.join(conf.DATA_PATH, "ostriz.json")
    return import_results(input_file)


@pytest.fixture(scope="module")
def config_cloudtp(config_prop):
    cloudtp = copy.deepcopy(config_prop)
    cloudtp["polarion-project-id"] = "CLOUDTP"
    cloudtp["cfme_parametrize"] = True
    return cloudtp


class TestParamE2E:
    def test_e2e_names_transform(self, config_cloudtp, records_names):
        exporter = XunitExport("5_8_0_17", records_names, config_cloudtp)
        complete = exporter.export()
        fname = "complete_parametrization.xml"
        with open(os.path.join(conf.DATA_PATH, fname), encoding="utf-8") as input_xml:
            parsed = input_xml.read()
        assert complete == parsed



"""
Imports.
"""

from dump2polarion.configuration import get_config
from dump2polarion.results.importer import import_results
from dump2polarion.exporters.requirements_exporter import RequirementExport
from dump2polarion.submit import submit_and_verify
from dump2polarion.exporters.testcases_exporter import TestcaseExport
from dump2polarion.exporters.xunit_exporter import XunitExport

__all__ = [
    "RequirementExport",
    "TestcaseExport",
    "XunitExport",
    "import_results",
    "get_config",
    "submit_and_verify",
]



# pylint: disable-all

import warnings

from dump2polarion.exporters.transform import *  # noqa

warnings.warn(
    "The transform module was moved to 'dump2polarion.exporters.transform'", DeprecationWarning
)
